const osrm = require('osrm');
const fs = require('fs');
const neatCsv = require('neat-csv');
const polyline = require('@mapbox/polyline');

exports.route2html = _route2html;

exports.csv = async (path, separator='\t', headers=null) => {
    return await neatCsv(await _readFile(path), separator=separator, headers=headers);
}

exports.getOSRM = (path,algorithm='MLD')=>{
    return new osrm({
        path: path,
        algorithm: algorithm,
    });
}

exports.osrmRoute = _osrmRoute;
function _osrmRoute(osrmClient, coords){
    return new Promise((resolve, reject) => {
        osrmClient.route({
            coordinates: coords
        }, function(err, result) {
            if (err) {
                reject(err)
                return
            }
            resolve(result);
        });
    });
}

exports.osrmTable = (osrmClient, coords, multiplier=1) =>{
    return new Promise((resolve, reject) => {
        osrmClient.table({
            coordinates: coords
        }, function(err, result) {
            if (err) {
                reject(err)
                return
            }
            let width = result.durations.length;
            let height = result.durations[0].length;
            for (let i = 0; i < width; i++) {
                for (let j = 0; j < height; j++) {
                    result.durations[i][j] = Math.floor(result.durations[i][j] * multiplier);
                }
            }
            resolve(result);
        });
    });
}

exports.shell = (cmd)  => {
    const exec = require('child_process').exec;
    return new Promise((resolve, reject) => {
        exec(cmd, (error, stdout, stderr) => {
            if (error) {
                console.warn(error);
                reject(error);
                return;
            }
            resolve(stdout ? stdout : stderr);
        });
    });
}

exports.readFile = _readFile;
exports.writeFile = _writeFile;

function _readFile(fileName, type='utf8'){
    return new Promise(function(resolve, reject) {
        fs.readFile(fileName, type, (err, data) => {
            err ? reject(err) : resolve(data);
        });
    });
}

function _writeFile(fileName, content){
    return new Promise(function(resolve, reject) {
        fs.writeFile(fileName, content, 'utf8', (err) => {
            if (err) {
                console.log(err);
                throw err;
            }
            resolve();
        });
    });
}

async function _route2html(route, osrmapi){
    let coords = [];
    route.steps.forEach((step, whocares) => {
        if (step.type != 'break'){
            coords.push(step.location);
        }
    });
    let geometry = (await _osrmRoute(osrmapi,coords)).routes[0].geometry;
    let decoded = polyline.decode(geometry);
    let pl= [];
    decoded.forEach((d,i) => {
        pl.push([d[1],d[0]]);
    });
    let features = [];
    coords.forEach((coord, index) => {
        features.push({
            type: 'Feature',
            properties:{
                label: `S${index}`,
                description: '',
                icon: 'shop',
            },
            geometry:{
                type: 'Point',
                coordinates: coord,
            },
        });
    })

    let template = await _readFile(path.resolve(__dirname, 'template.html'));
    template = template.replace('{{ data }}',JSON.stringify({
        places:{
            type: 'FeatureCollection',
            features: features,
        },
        path: pl,
    },null,4));
    return template;
}
